#ifndef CRNAFREEENERGYSINGLEMOVESTRUCTSTATE_HH_
#define CRNAFREEENERGYSINGLEMOVESTRUCTSTATE_HH_


#include "CRNAFreeEnergySingleMoveState.hh"
#include "RNAFreeEnergySingleMoveStructState.hh"

namespace ell
{	
		/**
		 * An object of this class represents a RNA free energy single move
		 * state with a common RNA sequence in a compressed representation.
		 * 
		 * @author Martin Mann, Sebastian Will, Andreas Richter
		 */
	class CRNAFreeEnergySingleMoveStructState
	 : public CRNAFreeEnergySingleMoveState
	{
	protected:
			/*! Returns the uncompressed common RNA sequence string.
			 */
		virtual std::string getUncompressedSeqStr() const;

			/*! Returns the uncompressed RNA structure string.
			 */
		virtual std::string getUncompressedStructStr() const;

		CRNAFreeEnergySingleMoveStructState()
		{}

	public:
		CRNAFreeEnergySingleMoveStructState(const Structure& struc);

		virtual ~CRNAFreeEnergySingleMoveStructState();

	// abstract functions (CState)
		virtual bool			operator== (const CState& cState2) const;
		virtual bool			operator!= (const CState& cState2) const;

		virtual CRNAFreeEnergySingleMoveStructState*	clone() const;

		virtual std::string	getCStateStringRepresentation() const;

			/*! Returns a specific std::string representation of the
			 *  uncompressed representation of this state.
			 * 
			 *  BRACKETNOTATION */
		virtual std::string	getUncomprStateStringRepresentation() const;

		virtual State* uncompressedClone() const;

		virtual void	uncompress(State& state) const;

		virtual CRNAFreeEnergySingleMoveStructState*
				  readBinaryInputClone(std::ifstream& source) const;

		virtual void	writeBinaryOutput(std::ofstream& destination) const;
	};

}

#endif /*CRNAFREEENERGYSINGLEMOVESTRUCTSTATE_HH_*/
