#ifndef PIVOTMOVESET_HH_
#define PIVOTMOVESET_HH_


#include "LatticeMoveSet.hh"

namespace ell
{
		/**
		 * This class handles pivot moves on LatticeProtein objects.
		 * The pivot moves are implemented as point mutations on the
		 * relative move string.
		 * 
		 * @author Martin Mann, Sebastian Will, Andreas Richter
		 */
	class PivotMoveSet : public LatticeMoveSet
	{
	public:
		PivotMoveSet(const LatticeModel* lattice); 

		virtual ~PivotMoveSet();

	// abstract functions (LatticeMoveSet)
			/*! Applies a pivot move to the given LatticeProtein on the 
			 * specified position and returns a new LatticeProtein object 
			 * as result.
			 * 
			 * @param todo the initial state
			 * @param position  the structure position where to apply the move
								which has to be larger than 0.
			 * @param moveIndex the move index (has to be less than the return 
			 * 					value of getMoveNumber())
			 * @return a new LatticeProtein object neighbored to the todo object
			 *
			 *  In case of invalid moves, the behaviour
			 *  of the program is not specified.
			 */
		virtual LatticeProtein* applyMove(const LatticeProtein* const todo, 
									 size_t position, size_t moveIndex) const;

			/*! Applies a pivot move to the given LatticeProtein on the 
			 * specified position INPLACE and returns the modified object
			 * as result.
			 *
			 * @param todo the initial state to modify
			 * @param position  the structure position where to apply the move
								which has to be larger than 0.
			 * @param moveIndex the move index (has to be less than the return 
			 * 					value of getMoveNumber())
			 * @return the modified LatticeProtein 
			 *
			 *  In case of invalid moves, the behaviour
			 *  of the program is not specified.
			 */
		virtual LatticeProtein* applyMoveInPlace(LatticeProtein* todo, size_t position,
									size_t moveIndex) const;
		
		virtual unsigned int getMoveNumber() const;
	};

} // namespace ell

#endif /*PIVOTMOVESET_HH_*/
