#include <ell/AllowedBasePairs.hh>
#include <ell/assertell.hh>

namespace ell
{

	AllowedBasePairs::AllowedBasePairs(	const Alphabet* _alph,
	 									const std::string& bps)
		: alph(_alph) {
			// add the allowed base pairings to basePairSet
		std::string base1, base2;
		size_t elemLength = alph->getElementLength();
		for (size_t i = 0; i < bps.length(); i += 2 * elemLength + 1) {	// +1 for comma
			base1 = bps.substr(i, elemLength);
			base2 = bps.substr(i + elemLength, elemLength);
			assertell(	(alph->isAlphabetString(base1)
			 			&& alph->isAlphabetString(base2)), 
			 			"The base pair "+base1+", "+base2
			 			+" is not part of the alphabet.");
			allowedBasePairSet.insert(BPair(alph->getElement(base1),
			 								alph->getElement(base2)));
		}
	}

	AllowedBasePairs::~AllowedBasePairs() {
	}

	bool 
	AllowedBasePairs::operator== (const AllowedBasePairs& abp2) const {
		return *alph == *(abp2.alph)
			&& allowedBasePairSet == abp2.allowedBasePairSet;
	}

	bool 
	AllowedBasePairs::operator!= (const AllowedBasePairs& abp2) const {
		return *alph != *(abp2.alph)
			&& allowedBasePairSet != abp2.allowedBasePairSet;
	}

	bool 
	AllowedBasePairs::allowedBasePair(const Alphabet::AlphElem& first,
					const Alphabet::AlphElem& second) const {
		return (allowedBasePairSet.find(BPair(first, second))
				!= allowedBasePairSet.end()
				|| allowedBasePairSet.find(BPair(second, first))
				!= allowedBasePairSet.end() );
	}

}
