#include <ell/Alphabet.hh>
#include <ell/assertell.hh>

namespace ell {
	
	Alphabet::Alphabet(	const std::string& alphabetString, 
						const unsigned int _elementLength)
	 : elementLength(_elementLength) 
	{
		assertell (alphabetString.length() % elementLength == 0,
	"length of the alphabetstring has to be multiple of the element length");
			
			 // initialisieren
		alph2string = std::vector<std::string>(	
							alphabetString.length() / elementLength, "");
		AlphElem actElem = 0;
		std::string actStr = "";
		for (std::string::size_type i = 0; i<alphabetString.length(); 
					i+= elementLength) {
			actStr = alphabetString.substr(i,elementLength);
			string2alph[actStr] = actElem;
			alph2string[actElem] = actStr;
			actElem++;
		}
	}

	Alphabet::Alphabet(const std::vector<std::string> & alphabetStrings) {

		unsigned int alphabetSize=alphabetStrings.size();

		assertell (alphabetSize>0,"No init of empty alphabet allowed");
		
		elementLength = alphabetStrings[0].length();
		
#ifdef NDEBUG
		{
			unsigned int i;
			for(i=0; i<alphabetSize; i++) {
				assertell(alphabetStrings[i].length() == elementLength, 
						  "All strings must have equal length");
			}
		}
#endif
		
		alph2string = std::vector<std::string>(alphabetSize, "");
		
		AlphElem actElem = 0;
		std::string actStr = "";
		for (std::string::size_type i = 0; i<alphabetSize; i++) {
			actStr = alphabetStrings[i];
			string2alph[actStr] = actElem;
			alph2string[actElem] = actStr;
			actElem++;
		}
	}

	Alphabet::~Alphabet() {		
	}

	bool			
	Alphabet::operator== (const Alphabet& alph2) const {
		return string2alph == alph2.string2alph
			&& alph2string == alph2.alph2string
			&& elementLength == alph2.elementLength;
	}

	bool			
	Alphabet::operator!= (const Alphabet& alph2) const {
		return string2alph != alph2.string2alph
			&& alph2string != alph2.alph2string
			&& elementLength != alph2.elementLength;
	}

	// string to sequence to string

	Alphabet::Sequence 
	Alphabet::getSequence(const std::string& seqString) const {
		assertell (seqString.length() % elementLength == 0,
			"the sequence length has to be a multiple of the element length");
			
		Alphabet::Sequence seq;
		for (std::string::size_type i=0; i<seqString.length(); 
					i+= elementLength) {
			seq.push_back(getElement(seqString.substr(i,elementLength)));
		}
		return seq;
	}
	
	std::string	
	Alphabet::getString(const Alphabet::Sequence& sequence) const {
		std::string str; 
		for (Alphabet::Sequence::const_iterator it = sequence.begin(); 
					it != sequence.end(); it++) {
			str += getString(*it);
		}
		return str;
	}
	
	std::string 
	Alphabet::getString(const Alphabet::AlphElem& elem) const {
		assertell( elem < alph2string.size(), 
			"elem is no valid element of the alphabet");
		return alph2string.at(elem);
	}

	Alphabet::AlphElem 
	Alphabet::getElement(const std::string& alphElemStr) const {
		STR2ALPH_TYPE::const_iterator member = string2alph.find(alphElemStr);
		assertell(member != string2alph.end(),
			"element '"+alphElemStr+"' is no member of the alphabet");
		return member->second;
	}

	bool	
	Alphabet::isAlphabetString(const std::string& str) const {
				// muss vielfaches der elementlaenge sein
		bool retVal = (str.length() % elementLength == 0);
		for (std::string::size_type i = 0; retVal && i< str.size(); 
				i+=elementLength) {
			retVal = 	string2alph.find(str.substr(i,elementLength)) 
						!= string2alph.end();
		}
		return retVal;
	}

	unsigned int 
	Alphabet::getIndex(const AlphElem& elem) const {
		return (unsigned int)elem;
	}
	
	unsigned int 
	Alphabet::getIndex(const std::string& elemStr) const {
		return getIndex(getElement(elemStr));
	}
	
	unsigned int 
	Alphabet::getElement(const unsigned int index) const {
		assertell(index < getAlphabetSize(), 
			"the index leaves the valid indices of the alphabet");
		return (AlphElem)index;
	}
	
} // namespace ell

