#include <ell/CRNAFreeEnergySingleMoveState.hh>
#include <fstream>	// for readBinaryInputClone(..), writeBinaryOutput(..)

namespace ell
{
	CRNAFreeEnergySingleMoveState::CRNAFreeEnergySingleMoveState(
	 const Sequence& seq, const Structure& struc)
	 : seqLength(seq.size()),
	   cSequence(compressSeq(seq, RNAFreeEnergy::getSequenceAlphabet())),
	   cStructure(compressSeq(struc, RNAFreeEnergy::getStructureAlphabet())) {
	}

	CRNAFreeEnergySingleMoveState::~CRNAFreeEnergySingleMoveState() {
	}

	std::string
	CRNAFreeEnergySingleMoveState::getUncompressedSeqStr() const {
		return uncompressSeq(cSequence, RNAFreeEnergy::getSequenceAlphabet(),
							   seqLength);
	}

	std::string
	CRNAFreeEnergySingleMoveState::getUncompressedStructStr() const {
		return uncompressSeq(cStructure,RNAFreeEnergy::getStructureAlphabet(),
							   seqLength);
	}

	bool	
	CRNAFreeEnergySingleMoveState::operator== (const CState& _cState2) const {
		CRNAFreeEnergySingleMoveState cState2 =
		 dynamic_cast<const CRNAFreeEnergySingleMoveState&>(_cState2);
		return (this->seqLength == cState2.seqLength
				  && this->cSequence == cState2.cSequence
				  && this->cStructure == cState2.cStructure);
	}

	bool
	CRNAFreeEnergySingleMoveState::operator!= (const CState& _cState2) const {
		CRNAFreeEnergySingleMoveState cState2 =
		 dynamic_cast<const CRNAFreeEnergySingleMoveState&>(_cState2);
		return (this->seqLength != cState2.seqLength
				  && this->cSequence != cState2.cSequence
				  && this->cStructure != cState2.cStructure);
	}

	CRNAFreeEnergySingleMoveState* 
	CRNAFreeEnergySingleMoveState::clone() const {
		return new CRNAFreeEnergySingleMoveState(*this);
	}

	std::string
	CRNAFreeEnergySingleMoveState::getCStateStringRepresentation() const {
		std::string strRepr;
		for (size_t i = 0; i < cSequence.size(); i++) {
			strRepr += cSequence[i];
		}
		for (size_t i = 0; i < cStructure.size(); i++) {
			strRepr += cStructure[i];
		}
		return strRepr;
	}

	std::string
	CRNAFreeEnergySingleMoveState::getUncomprStateStringRepresentation()
	 const {
		return ( getUncompressedStructStr() + "(" +
				  getUncompressedSeqStr() +")" );
	}

	State*
	CRNAFreeEnergySingleMoveState::uncompressedClone() const {
		return new RNAFreeEnergySingleMoveState(getUncompressedSeqStr(),
												  getUncompressedStructStr());
	}

	void
	CRNAFreeEnergySingleMoveState::uncompress(State& state) const {
		dynamic_cast<RNAFreeEnergySingleMoveState&>(state) =
		 RNAFreeEnergySingleMoveState(getUncompressedSeqStr(),
									  getUncompressedStructStr());
	}

	CRNAFreeEnergySingleMoveState*
	CRNAFreeEnergySingleMoveState::readBinaryInputClone(std::ifstream& source)
	 const {

			// create new CRNAFreeEnergySingleMoveState
		CRNAFreeEnergySingleMoveState* cState =
		 new CRNAFreeEnergySingleMoveState();

			// read seqLength
		source.read(reinterpret_cast<char*>(&(cState->seqLength)),
						   sizeof(cState->seqLength));

		size_t size;

			// read cSequence
		source.read(reinterpret_cast<char*>(&size), sizeof(size));
		cState->cSequence.resize(size);
		for (size_t i = 0; i < cState->cSequence.size(); i++) {
			source.read(reinterpret_cast<char*>(&(cState->cSequence[i])),
						  sizeof(cState->cSequence[i]));
		}

			// read cStructure
		source.read(reinterpret_cast<char*>(&size), sizeof(size));
		cState->cStructure.resize(size);
		for (size_t i = 0; i < cState->cStructure.size(); i++) {
			source.read(reinterpret_cast<char*>(&(cState->cStructure[i])),
						  sizeof(cState->cStructure[i]));
		}

		return cState;
	}

	void
	CRNAFreeEnergySingleMoveState::writeBinaryOutput(std::ofstream& destination)
	 const {
			// write seqLength
		destination.write(
	 	 reinterpret_cast<const char*>(&seqLength), sizeof(seqLength));

		size_t size;

			// write cSequence
		size = cSequence.size();
		destination.write(reinterpret_cast<const char*>(&size), sizeof(size));
		for (size_t i = 0; i < size; i++) {
			destination.write(reinterpret_cast<const char*>(&(cSequence[i])),
							   sizeof(cSequence[i]));
		}

			// write cStructure
		size = cStructure.size();
		destination.write(reinterpret_cast<const char*>(&size), sizeof(size));
		for (size_t i = 0; i < size; i++) {
			destination.write(reinterpret_cast<const char*>(&(cStructure[i])),
							   sizeof(cStructure[i]));
		}
	}

} // namespace ell
