#include <ell/PivotMoveSet.hh>

namespace ell
{
	PivotMoveSet::PivotMoveSet(const LatticeModel* lattice)
	 : LatticeMoveSet(lattice) 
	{} 

	PivotMoveSet::~PivotMoveSet()
	{}

// abstract functions (LatticeMoveSet)
	LatticeProtein*
	PivotMoveSet::applyMove(const LatticeProtein* const todo, 
	 size_t position, size_t moveIndex) const {

			// apply the pivot move inplace to a copy of todo
		return applyMoveInPlace(new LatticeProtein(*todo), position,
				 moveIndex);
	}

	LatticeProtein* 
	PivotMoveSet::applyMoveInPlace(LatticeProtein* todo, 
	 size_t position, size_t moveIndex) const {

		assertell(moveIndex >=0 && moveIndex < getMoveNumber(),
	"The pivot move moveIndex has to be in [0, number of available moves).");
		assertell(position >0 && position < todo->relMoves->size(),
	"The pivot move position has to be in [1, structure length).");

			// get the mutated relative move
		Move mutatedMove = lattice->getDescriptor()->getNeighborhood().
								 getElementByIndex(moveIndex).getMove();

			// if the current relative move at position is equal to the chosen
			// mutated move, choose the move with the highest move index
		if (todo->getRelativeMoves()[position] == mutatedMove) {
			mutatedMove = lattice->getDescriptor()->getNeighborhood().
								 getElementByIndex(getMoveNumber()).getMove();
		}

			// change the relative move string at position to the mutated move
		(*(todo->relMoves))[position] = mutatedMove;

		return todo;
	}

	unsigned int
	PivotMoveSet::getMoveNumber() const {
			// number of pivot moves is the number of relative moves minus
			// the current relative move
		return lattice->getDescriptor()->getNeighborhood().size() - 1;
	}

}
